const EXPORTED_SYMBOLS = [""];

const Cc = Components.classes;
const Ci = Components.interfaces;

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
Components.utils.import("resource://hltmnn-twitter-modules/util.js");

const httpObservers = {

  response: {
    QueryInterface: XPCOMUtils.generateQI([Ci.nsIObserver]),

    // nsIObserver
    observe: function(aSubject, aTopic, aData) {
      var httpChannel = aSubject.QueryInterface(Ci.nsIHttpChannel);
      var win = fromTwitter(httpChannel);
      if (win === null) {
        //util.log("fromTwitter " + httpChannel.URI.spec);
        return;
      }

      if (httpChannel.contentType.indexOf("image/") !== 0) {
        util.log("ct " + httpChannel.contentType + "\n"+httpChannel.URI.spec);
        return;
      }



      if (win.document.getUserData("twitter-skip-blocking") === "ok") {
        return;
      }

      if (httpChannel.contentLength < Prefs.bgMax) {
        return;
      }


      util.log("cancel " + httpChannel.contentLength + "\n" + httpChannel.URI.spec);
      httpChannel.cancel(Components.results.NS_ERROR_ABORT);

      var style = win.getComputedStyle(win.document.querySelector("body"), null);
      var userData = {
        "image":      style.getPropertyValue("background-image"),
        "repeat":     style.getPropertyValue("background-repeat"),
        "attachment": style.getPropertyValue("background-attachment")
      };

      if (userData["image"].indexOf("url(data:") === 0) {
        return;
      }

      if (("url(" + httpChannel.URI.spec + ")") !== userData["image"]) {
        util.log("???  " + userData["image"] + "\n" + httpChannel.URI.spec);
        return;
      }

      win.document.setUserData("twitter-cancel2", JSON.stringify(userData), null);

      var s = win.document.body.style;
      s.backgroundImage = "url(" + buildImg(win.document, httpChannel.contentLength) + ")";
      s.backgroundAttachment = "scroll";
      s.backgroundRepeat= "no-repeat";

      win.document.addEventListener("click", function(evt) {
        if (evt.button !== 0 || evt.target.tagName !== "BODY") {
          return;
        }
        var doc = evt.target.ownerDocument;
        var userData = JSON.parse(doc.getUserData("twitter-cancel2"));
        doc.setUserData("twitter-skip-blocking", "ok", null);

        var s = doc.body.style;
        s.backgroundRepeat = userData["repeat"];
        s.backgroundImage = userData["image"];
        s.backgroundAttachment = userData["attachment"];
      }, false)
    }
  }
};


function fromTwitter(httpChannel) {
  var domWindow = HttpContext.findWindow(httpChannel);
  if (domWindow === null) {
    util.log("no win " + httpChannel.URI.spec);
    return null;
  }

  var uri = domWindow.document.documentURIObject;
  switch (uri.scheme) {
    case "http":
      break;
    default:
      return null;
  }

  if (uri.host !== "twitter.com") {
    //util.log("e " + domWindow.document.documentURIObject.spec + "\n" + httpChannel.URI.spec);
    return null;
  }

  return domWindow;
}


function buildImg(doc, size) {
  var cnv = doc.createElement("canvas");
  cnv.width = 200;
  cnv.height = 400;


  var ctx = cnv.getContext("2d");

  ctx.rotate(-70 * Math.PI / 180);
  ctx.font = "bold 2.3em Verdana, Helvetica, sans-serif";
  ctx.fillStyle = "white";
  ctx.shadowOffsetX = 0;
  ctx.shadowOffsetY = 0;
  ctx.shadowBlur = 7;
  ctx.shadowColor = "black";

  var txt = util.getText("blocked");
  var w = ctx.measureText(txt).width;
  ctx.fillText(txt, -w/2 - 170, 140);

  var txt = util.getText("click", Math.round(size / 1024));
  var w = ctx.measureText(txt).width;
  ctx.fillText(txt, -w/2 - 170, 170);

  return cnv.toDataURL("image/png");
}


var Prefs = {
  bgMax: 0,
  _branch: null,

  init: function() {
    this._branch = Cc["@mozilla.org/preferences-service;1"]
                    .getService(Ci.nsIPrefService)
                    .getBranch("extensions.twitter@hultmann.");
    this._branch.QueryInterface(Ci.nsIPrefBranch2);
    this._branch.addObserver("", this, false);
    this.update();
  },

  update: function() {
    this.bgMax = this._branch.getIntPref("web.backgroundBlocker.size") * 1024;
    util.log('pref! ' + this.bgMax);
  },

  QueryInterface: XPCOMUtils.generateQI([Ci.nsIObserver]),
  observe: function(aSubject, aTopic, aData) {
    this.update();
  }

};


Prefs.init();
util.setHttpObservers.enable(httpObservers.response);
